import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.RecursiveTask;

class ValueSumCounter extends RecursiveTask<Long> {
	private final Node node;

	public ValueSumCounter(Node node) {
		this.node = node;
	}

	@Override
	protected Long compute() {
		if (node.depth() < 10) {
			return Node.countSum(node);
		}
		
		long sum = node.getValue();
		List<ValueSumCounter> subTasks = new LinkedList<>();

		for (Node child : node.getChildren()) {
			ValueSumCounter task = new ValueSumCounter(child);
			task.fork(); //  
			subTasks.add(task);
		}

		for (ValueSumCounter task : subTasks) {
			sum += task.join(); //      
		}

		return sum;
	}
}

public class TestForkJoin {

	public static void main(String[] args) {
		Node root = BasicNode.createTree(22);	
		
		System.out.println("created");
		
		long t = System.currentTimeMillis();	
		System.out.println(new ForkJoinPool(4).invoke(new ValueSumCounter(root)));
		System.out.println(System.currentTimeMillis() - t);

	}

}
